

/**************************************************************************************************
** Create tables
**************************************************************************************************/
USE [DBA]
GO
if	exists(select 1 from dbo.sysobjects WHERE type = 'D'and object_name(parent_obj) = 'Deadlock_Report_Main')
	BEGIN
		declare	@defaultName	varchar(256)	,
			@query		varchar(1024)
		SELECT	@defaultName = name
		FROM	dbo.sysobjects
		WHERE	type = 'D'
			and object_name(parent_obj) = 'Deadlock_Report_Main'

		set	@query = 'ALTER	TABLE [dbo].[Deadlock_Report_Main] DROP CONSTRAINT ' + quotename(@defaultName)

		exec	(@query)
	END
GO
IF 	EXISTS (SELECT 1 FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Deadlock_Report_Main_Deadlock_Report_Detail]') AND parent_object_id = OBJECT_ID(N'[dbo].[Deadlock_Report_Detail]'))
	ALTER	TABLE [dbo].[Deadlock_Report_Detail] DROP CONSTRAINT [FK_Deadlock_Report_Main_Deadlock_Report_Detail]
GO
IF 	EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Deadlock_Report_Main]') AND type in (N'U'))
	DROP	TABLE [dbo].[Deadlock_Report_Main]
GO
CREATE	TABLE [dbo].[Deadlock_Report_Main]
	(
	[drmID]		[int] IDENTITY(1,1) NOT NULL	,
	[event_data]	[xml]				,
	[module_guid]	[uniqueidentifier]		,
	[package_guid]	[uniqueidentifier]		,
	[drmDate]	[datetime2](7)			,
	[file_name]	[varchar](256)			,
	[file_offset]	[int] NULL			,
	[File_Exists]	[int] NULL			,
	CONSTRAINT [PK_Deadlock_Report_Main_drmID] PRIMARY KEY CLUSTERED 
		(
		[drmID]
		)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
	) ON [PRIMARY]
GO
ALTER	TABLE [dbo].[Deadlock_Report_Main] ADD  DEFAULT (getdate()) FOR [drmDate]
GO

IF	EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Deadlock_Report_Detail]') AND type in (N'U'))
	DROP	TABLE [dbo].[Deadlock_Report_Detail]
GO
CREATE	TABLE [dbo].[Deadlock_Report_Detail]
	(
	[drmID]			[int]		,
	[PagelockObject]	[varchar](200)	,
	[DeadlockObject]	[varchar](200)	,
	[Victim]		[int]		,
	[Procedure]		[varchar](200)	,
	[LockMode]		[varchar](10)	,
	[Code]			[varchar](1000)	,
	[ClientApp]		[varchar](100)	,
	[HostName]		[varchar](20)	,
	[LoginName]		[varchar](20)	,
	[TransactionTime]	[datetime]	,
	[InputBuffer]		[varchar](max)	,
	[obj_name]		[varchar](1024)	,
	[lines]			[varchar](1024)	,
	[processID]		[varchar](50)	,
	[proc_name]		[varchar](800)	,
	dbname			[varchar](256)
	) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]

GO
ALTER	TABLE [dbo].[Deadlock_Report_Detail]  WITH CHECK ADD  CONSTRAINT [FK_Deadlock_Report_Main_Deadlock_Report_Detail] FOREIGN KEY([drmID])
REFERENCES [dbo].[Deadlock_Report_Main] ([drmID])
GO
ALTER	TABLE [dbo].[Deadlock_Report_Detail] CHECK CONSTRAINT [FK_Deadlock_Report_Main_Deadlock_Report_Detail]
GO
IF 	EXISTS(SELECT 1 FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Deadlock_Report_Detail]') AND name = N'IX_Deadlock_Report_Detail_drmID')
	DROP	INDEX [IX_Deadlock_Report_Detail_drmID] ON [dbo].[Deadlock_Report_Detail] WITH ( ONLINE = OFF )
GO
CREATE	CLUSTERED INDEX [IX_Deadlock_Report_Detail_drmID] ON [dbo].[Deadlock_Report_Detail]([drmID]) ON [PRIMARY]
GO
IF	EXISTS (SELECT 1 FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Deadlock_Report_Main]') AND name = N'IX_Deadlock_Report_Main_drmDate')
	DROP INDEX [IX_Deadlock_Report_Main_drmDate] ON [dbo].[Deadlock_Report_Main] WITH ( ONLINE = OFF )
GO
CREATE	INDEX [IX_Deadlock_Report_Main_drmDate] ON [dbo].[Deadlock_Report_Main]([drmDate])
INCLUDE	([event_data]) ON [PRIMARY]
GO
IF	EXISTS (SELECT 1 FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Deadlock_Report_Main]') AND name = N'IX_Deadlock_Report_Main_File_Exists')
	DROP	INDEX [IX_Deadlock_Report_Main_File_Exists] ON [dbo].[Deadlock_Report_Main] WITH ( ONLINE = OFF )
GO
CREATE	INDEX [IX_Deadlock_Report_Main_File_Exists] ON [dbo].[Deadlock_Report_Main]([File_Exists])
INCLUDE	([file_name],[file_offset]) ON [PRIMARY]
GO
IF 	EXISTS (SELECT 1 FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Deadlock_Report_Main]') AND name = N'IX_Deadlock_Report_Main_module_guid')
	DROP	INDEX [IX_Deadlock_Report_Main_module_guid] ON [dbo].[Deadlock_Report_Main] WITH ( ONLINE = OFF )
GO
CREATE	INDEX [IX_Deadlock_Report_Main_module_guid] ON [dbo].[Deadlock_Report_Main]([module_guid],[package_guid]) ON [PRIMARY]
GO
IF	EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[general_configuration]') AND type in (N'U'))
	DROP	TABLE [dbo].[general_configuration]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

SET ANSI_PADDING ON
GO

CREATE	TABLE [dbo].[general_configuration](
	[configuration_desc]	[varchar](64) NOT NULL		,
	[configuration_value]	[varchar](2048) NOT NULL	,
CONSTRAINT [PK_general_configuration_configuration_desc] PRIMARY KEY CLUSTERED 
	(
	[configuration_desc] ASC
	)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
INSERT	INTO [general_configuration]([configuration_desc],[configuration_value])
--------------------------------------------------------------------
-- CHANGE THE XE PATH TO SOMETHING OF YOUR CHOOSING
--------------------------------------------------------------------
VALUES	('deadlock_xml_root', 'c:\record_deadlocks\blocked_process')
--------------------------------------------------------------------
-- CHANGE THE XE PATH TO SOMETHING OF YOUR CHOOSING
--------------------------------------------------------------------
GO
/**************************************************************************************************
** Create extended event
**************************************************************************************************/
USE	DBA
go


if	exists
	(select	1 from	sys.dm_xe_sessions where name = 'blocked_process')
	drop	EVENT SESSION [blocked_process] ON SERVER
GO
declare	@eeQuery	varchar(2048)	,
	@base_loc	varchar(256)

select	@base_loc = configuration_value
from	general_configuration

set	@eeQuery = '
CREATE	EVENT SESSION [blocked_process] ON SERVER
	ADD	EVENT sqlserver.xml_deadlock_report
		(
		ACTION	(
			sqlserver.client_app_name,
			sqlserver.client_hostname,
			sqlserver.database_id
			)
		)
ADD	TARGET package0.asynchronous_file_target
	(
	SET	filename		= N' + char(39) + @base_loc + '.xel''		,
		metadatafile		= N' + char(39) + @base_loc + '.xem''		,
		max_file_size		= (4096)	,
		max_rollover_files	= 2
	)
WITH	(MAX_DISPATCH_LATENCY = 5SECONDS);'

exec	(@eeQuery)

GO
EXEC sp_configure 'show advanced options', 1 ;
GO
RECONFIGURE with override;
GO
/* Enabled the blocked process report */
EXEC sp_configure 'blocked process', '5';
GO
RECONFIGURE with override;
GO
/* Start the Extended Events session */
ALTER EVENT SESSION [blocked_process] ON SERVER
STATE = start;
GO
SET ANSI_PADDING OFF
GO
/**************************************************************************************************
** Create file exists function
**************************************************************************************************/
USE [DBA]
GO
IF 	EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ufnFileExists]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
	DROP FUNCTION [dbo].[ufnFileExists]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO
/*************************************************************************************************
**  purpose:
**	scalar valued function to check if file exists
**  author:
**      William J. Wolf
**
**  date created:
**      2014-09-11
**
**  parameter description:
	@fileName

**  date modified/reason:
**
**  Notes:
DISCLAIMER:  This is written by William Wolf(@sqlwarewolf).  William is not responsible for any
adverse affects this may cause.  Always test thoroughly on a development system before implementing in production
**************************************************************************************************************/
create	function [dbo].[ufnFileExists]
	(
	@fileName	nvarchar(260)
	)
returns	int
as
BEGIN
	declare	@File_Exists int
	EXEC	Master.dbo.xp_fileexist @fileName, @File_Exists OUT
	return	@File_Exists
END
		
GO
USE [DBA]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
if	object_id('uspRecordDeadlocks') is not null
	drop	procedure uspRecordDeadlocks
GO
/*************************************************************************************************
**  purpose:
**	Read deadlock extended event xml and record the information
**  author:
**      William J. Wolf
**
**  date created:
**      2014-09-11
**
**  parameter description:
@lastOffset

**  date modified/reason:
**
**  Notes:
DISCLAIMER:  This is written by William Wolf(@sqlwarewolf).  William is not responsible for any
adverse affects this may cause.  Always test thoroughly on a development system before implementing in production

delete deadlock_report_main
**************************************************************************************************************/
create	procedure [dbo].[uspRecordDeadlocks]
	@lastOffset	bit = 1
as
set	quoted_identifier on
set	nocount on
/*********************************************************************************************
**
*********************************************************************************************/

	declare	@initial_file_name	nvarchar(60)	,
		@initial_offset		bigint		,
		@drmID			int		,
		@deadlock_xml_root	nvarchar(260)	,
		@filename		nvarchar(260)	,
		@metadatafile		nvarchar(260)
	

	if	object_id('tempdb..#tmpProcess') is not null
		drop	table #tmpProcess
		
	create	table #tmpProcess
		(
		ProcessID		varchar(50)		,
		Victim			varchar(50)		,
		PageLockObject		varchar(200)		,
		DeadLockObject		varchar(200)		,
		[Procedure]		varchar(800)		,
		obj_name		varchar(1500)		,
		LockMode		varchar(10)		,
		Code			varchar(1000)		,
		ClientApp		varchar(100)		,
		HostName		varchar(20)		,
		LoginName		varchar(20)		,
		TransactionTime		datetime		,
		InputBuffer		varchar(max)		,
		associatedObjectID	bigint			,
		[dbid]			int			,
		fileid			int			,
		pageid			int			,
		line			int			,
		stmtstart		int			,
		stmtend			int			,
		sqlhandle		varchar(1000)		,
		drmID			int			,
		tmpProcessID		int identity(1,1)
		)

	create	clustered index #ix#tmpProcess on #tmpProcess(drmID, tmpProcessID)


	if	object_id('tempdb..#tmpLine') is not null
		drop	table #tmpLine

	create	table #tmpLine
		(
		drmID		int		,
		victim		varchar(50)	,
		inputBuffer	varchar(max)	,
		processID	varchar(50)	,
		LineList	varchar(512)
		)
		
	create	clustered index #ix#tmpLine on #tmpLine(drmID)
		
	
	select	@deadlock_xml_root = configuration_value
	from	general_configuration
	where	configuration_desc = 'deadlock_xml_root'

	select	@filename		= @deadlock_xml_root + '*.xel'	,
		@metadatafile		= @deadlock_xml_root + '*.xem'



	if	OBJECT_ID('tempdb..#cte', 'U') is not null
		drop	table #cte;
		
	create	table #cte
		(
		DeadlockID	INT	,
		DeadlockGraph	XML
		);
		
	create	clustered index #ix#cte on #cte(DeadlockID);

	if	OBJECT_ID('tempdb..#Victims', 'U') is not null
		drop	table #Victims;
		
	create	table #Victims
		(
		ID		varchar(50)	,
		DeadlockID	int
		);
		
	create	clustered index #ix#Victims on #Victims(DeadlockID);

	if	OBJECT_ID('tempdb..#Locks', 'U') is not null
		drop	table #Locks;
		
	create	table #Locks
		(
		DeadlockID		int		,
		LockID			varchar(100)	,
		LockProcessId		varchar(200)	,
		LockEvent		varchar(8000)	,
		ObjectName		varchar(256)	,
		LockMode		varchar(10)	,
		Database_id		int		,
		AssociatedObjectId	bigint		,
		WaitType		varchar(100)	,
		WaitProcessId		varchar(200)	,
		WaitMode		varchar(10)
		);
		
	create	clustered index #ix#Locks on #Locks(DeadlockID);
		
	if	OBJECT_ID('tempdb..#Lines', 'U') is not null
		drop	table #Lines;
		
	create	table #Lines
		(
		DeadlockID		int		,
		ProcessID		varchar(50)	,
		line			int		,
		stmtstart		int		,
		stmtend			int		,
		sqlhandle		varchar(1000)	,
		PageLockObject		varchar(200)	,
		DeadLockObject		varchar(200)	,
		[Procedure]		varchar(800)	,
		Code			varchar(max)	,
		[DBID]			int		,
		fileID			int		,
		pageID			int		,
		associatedObjectID	bigint
		);
		
	create	clustered index #ix#Lines on #Lines(DeadlockID);
		
	if	OBJECT_ID('tempdb..#Process', 'U') is not null
		drop	table #Process;
		
	create	table #Process
		(
		DeadlockID	int		,
		Victim		bit		,
		LockMode	varchar(10)	,
		ProcessID	varchar(50)	,
		KPID		int		,
		SPID		int		,
		SBID		int		,
		ECID		int		,
		IsolationLevel	varchar(200)	,
		WaitResource	varchar(200)	,
		LogUsed		int		,
		ClientApp	varchar(100)	,
		HostName	varchar(20)	,
		LoginName	varchar(20)	,
		TransactionTime	datetime	,
		BatchStarted	datetime	,
		BatchCompleted	datetime	,
		InputBuffer	varchar(max)	,
		DeadlockGraph	xml		,
		ExecutionStack	xml		,
		QueryStatement	varchar(max)	,
		ProcessQty	int		,
		TranCount	int
		);
		
	create	clustered index #ix#Process on #Process(DeadlockID);

	update	d
	set	d.file_exists = dbo.ufnFileExists([file_name])
	from	deadlock_report_main d
	where	file_exists = 1
		or file_exists is null

	select	@drmID = max(drmID)
	from	Deadlock_Report_Main
	
	if	@lastOffset = 1
		BEGIN
			select	@initial_file_name	= [file_name]	,
				@initial_offset		= file_offset
			from	Deadlock_Report_Main
			where	drmID = @drmID
				and (file_exists = 1)
		END	
/*********************************************************************************************
**
*********************************************************************************************/

	
	insert	into Deadlock_Report_Main(event_data, module_guid, package_guid, [file_name], file_offset)
	output	inserted.drmID, inserted.event_data
	into	#cte(DeadlockID, DeadlockGraph)
	select	f.event_data, f.module_guid, f.package_guid, f.[file_name], f.file_offset
	FROM    sys.fn_xe_file_target_read_file
			(
			@filename			,
			@metadatafile			,
			@initial_file_name		,
			@initial_offset	
			) f
		left join Deadlock_Report_Main drm on drm.module_guid = f.module_guid
			and drm.package_guid = f.package_guid
			and drm.[file_name] = f.[file_name]
			and drm.file_offset = f.file_offset
	where	drm.drmID is null



/*************************************************************************************************
**
*************************************************************************************************/
	--select DeadlockGraph as TargetData, DeadlockID from #cte

	--declare	@deadlockID int

	--declare	c cursor for
	--select	deadlockID
	--from	#cte
	--order	by deadlockID
	----
	--open	c
	----
	--fetch	next from c into @deadlockID
	--	while	@@FETCH_STATUS!= -1 
	--		BEGIN

	--		print @deadlockID
	--update	x
	--set	x.DeadlockGraph = CAST (
	--	    XEventData.XEvent.value ('(data/value)[1]', 'varchar(max)')
	--    AS XML) 
	--FROM	(select DeadlockGraph as TargetData, DeadlockID from #cte where deadlockID = @deadlockID) as Data
	--	CROSS APPLY TargetData.nodes ('//event') AS XEventData (XEvent)
	--	inner join #cte x on x.DeadlockID = Data.DeadlockID
	--		and x.DeadlockID = @deadlockID


	--			print @deadlockID
	--		END
	--deallocate c


	--	update	x
	--set	x.event_data = CAST (
	--	    XEventData.XEvent.value ('(data/value)[1]', 'varchar(max)')
	--    AS XML) 
	--FROM	(select event_data as TargetData, drmID from deadlock_report_main where drmID not in (170)) as Data
	--	CROSS APPLY TargetData.nodes ('//event') AS XEventData (XEvent)
	--	inner join deadlock_report_main x on x.drmID = Data.drmID
	
	
		
	--return

	--update	m
	--set	m.event_data = x.DeadlockGraph
	--from	deadlock_report_main m
	--	inner join #CTE x on x.DeadlockID = m.drmID





	insert	into #Lines
		(
		DeadlockID	,
		ProcessID	,
		line		,
		stmtstart	,
		stmtend		,
		sqlhandle	,
		PagelockObject	,
		DeadlockObject	,
		Code		,
		[Procedure]	,
		[DBID]		,
		fileID		,
		pageID		,
		associatedObjectID
		)
	select	DeadlockID,
		ProcessID		= Deadlock.Process.value('../../@id', 'varchar(50)')						,
		line			= Deadlock.Process.value('@line', 'int')							,
		stmtstart		= Deadlock.Process.value('@stmtstart', 'int')							,
		stmtend			= Deadlock.Process.value('@stmtend', 'int')							,
		sqlhandle		= Deadlock.Process.value('@sqlhandle', 'varchar(1000)')						,
		[PagelockObject]	= DeadlockGraph.value('/event[1]/data[1]/value[1]/deadlock[1]/resource-list[1]/pagelock[1]/@objectname', 'varchar(200)')	,
		[DeadlockObject]	= DeadlockGraph.value('/event[1]/data[1]/value[1]/deadlock[1]/resource-list[1]/objectlock[1]/@objectname', 'varchar(200)'),
		[Procedure]		= Deadlock.Process.value('../../executionStack[1]/frame[1]/@procname[1]', 'varchar(200)')	,
		Code			= Deadlock.Process.value('@code', 'varchar(MAX)')						,
		[dbid]			= COALESCE(
		Deadlock.Process.value('/event[1]/data[1]/value[1]/deadlock[1]/resource-list[1]/keylock[1]/@dbid', 'int')				,
		Deadlock.Process.value('/event[1]/data[1]/value[1]/deadlock[1]/resource-list[1]/pagelock[1]/@dbid', 'int')			,
		Deadlock.Process.value('/event[1]/data[1]/value[1]/deadlock[1]/process-list[1]/process[1]/@currentdb', 'int'))			,
		[fileid]		= Deadlock.Process.value('/event[1]/data[1]/value[1]/deadlock[1]/resource-list[1]/pagelock[1]/@fileid', 'int')		,
		[pageid]		= Deadlock.Process.value('/event[1]/data[1]/value[1]/deadlock[1]/resource-list[1]/pagelock[1]/@fileid', 'int')		,
		[associatedObjectID]	= Deadlock.Process.value('/event[1]/data[1]/value[1]/deadlock[1]/resource-list[1]/pagelock[1]/@associatedObjectId', 'bigint')
	FROM	#CTE CTE
		CROSS APPLY DeadlockGraph.nodes('/event/data/value/deadlock/process-list/process/executionStack/frame')  AS Deadlock (Process)
	order	by 1
	
/*************************************************************************************************
**
*************************************************************************************************/
	insert	into #Victims
		(
		ID		,
		DeadlockID
		)
	SELECT	ID = Victims.List.value('@id', 'varchar(50)'),
		CTE.DeadlockID
	FROM	#CTE AS CTE
		CROSS APPLY CTE.DeadlockGraph.nodes('//event/data/value/deadlock/victim-list/victimProcess') AS Victims (List)

/*************************************************************************************************
**
*************************************************************************************************/
	insert	into #Locks
		(
		DeadlockID		,
		LockID			,
		LockProcessId		,
		LockEvent		,
		ObjectName		,
		LockMode		,
		Database_id		,
		AssociatedObjectId	,
		WaitType		,
		WaitProcessId		,
		WaitMode
		)
	SELECT  CTE.DeadlockID,
		MainLock.Process.value('@id', 'varchar(100)') AS LockID,
		OwnerList.Owner.value('@id', 'varchar(200)') AS LockProcessId,
		REPLACE(MainLock.Process.value('local-name(.)', 'varchar(100)'), 'lock', '') AS LockEvent,
		MainLock.Process.value('@objectname', 'sysname') AS ObjectName,
		OwnerList.Owner.value('@mode', 'varchar(10)') AS LockMode,
		MainLock.Process.value('@dbid', 'INTEGER') AS Database_id,
		MainLock.Process.value('@associatedObjectId', 'BIGINT') AS AssociatedObjectId,
		MainLock.Process.value('@WaitType', 'varchar(100)') AS WaitType,
		WaiterList.Owner.value('@id', 'varchar(200)') AS WaitProcessId,
		WaiterList.Owner.value('@mode', 'varchar(10)') AS WaitMode
	FROM    #CTE CTE
		CROSS APPLY CTE.DeadlockGraph.nodes('//event/data/value/deadlock/resource-list') AS Lock (list)
		CROSS APPLY Lock.list.nodes('*') AS MainLock (Process)
		OUTER APPLY MainLock.Process.nodes('owner-list/owner') AS OwnerList (Owner)
		CROSS APPLY MainLock.Process.nodes('waiter-list/waiter') AS WaiterList (Owner)
        
/*************************************************************************************************
**
*************************************************************************************************/
	insert	into #Process
		(
		DeadlockID	,
		Victim		,
		LockMode	,
		ProcessID	,
		KPID		,
		SPID		,
		SBID		,
		ECID		,
		IsolationLevel	,
		WaitResource	,
		LogUsed		,
		ClientApp	,
		HostName	,
		LoginName	,
		TransactionTime	,
		BatchStarted	,
		BatchCompleted	,
		InputBuffer	,
		DeadlockGraph	,
		ExecutionStack	,
		QueryStatement	,
		ProcessQty	,
		TranCount
		)
	SELECT  CTE.DeadlockID,
		[Victim] = CONVERT(BIT, CASE WHEN Deadlock.Process.value('@id', 'varchar(50)') = ISNULL(Deadlock.Process.value('../../@victim', 'varchar(50)'), v.ID) 
					     THEN 1
					     ELSE 0
					END),
		[LockMode] = Deadlock.Process.value('@lockMode', 'varchar(10)'), -- how is this different from in the resource-list section?
		[ProcessID] = Process.ID, --Deadlock.Process.value('@id', 'varchar(50)'),
		[KPID] = Deadlock.Process.value('@kpid', 'int'), -- kernel-process id / thread ID number
		[SPID] = Deadlock.Process.value('@spid', 'int'), -- system process id (connection to sql)
		[SBID] = Deadlock.Process.value('@sbid', 'int'), -- system batch id / request_id (a query that a SPID is running)
		[ECID] = Deadlock.Process.value('@ecid', 'int'), -- execution context ID (a worker thread running part of a query)
		[IsolationLevel] = Deadlock.Process.value('@isolationlevel', 'varchar(200)'),
		[WaitResource] = Deadlock.Process.value('@waitresource', 'varchar(200)'),
		[LogUsed] = Deadlock.Process.value('@logused', 'int'),
		[ClientApp] = Deadlock.Process.value('@clientapp', 'varchar(100)'),
		[HostName] = Deadlock.Process.value('@hostname', 'varchar(20)'),
		[LoginName] = Deadlock.Process.value('@loginname', 'varchar(20)'),
		[TransactionTime] = Deadlock.Process.value('@lasttranstarted', 'datetime'),
		[BatchStarted] = Deadlock.Process.value('@lastbatchstarted', 'datetime'),
		[BatchCompleted] = Deadlock.Process.value('@lastbatchcompleted', 'datetime'),
		--[InputBuffer] = Input.Buffer.query('.', 'varchar(max)'),
		Deadlock.Process.value('inputbuf[1]', 'varchar(max)'),
		CTE.[DeadlockGraph],
		es.ExecutionStack,
		[QueryStatement] = Execution.Frame.value('.', 'varchar(max)'),
		ProcessQty = SUM(1) OVER (PARTITION BY CTE.DeadlockID),
		TranCount = Deadlock.Process.value('@trancount', 'int')
	FROM    #CTE CTE
		CROSS APPLY CTE.DeadlockGraph.nodes('//event/data/value/deadlock/process-list/process') AS Deadlock (Process)
		CROSS APPLY (SELECT Deadlock.Process.value('@id', 'varchar(50)') ) AS Process (ID)
		LEFT JOIN #Victims v ON Process.ID = v.ID
			and v.DeadlockID = CTE.DeadlockID
		CROSS APPLY Deadlock.Process.nodes('inputbuf') AS Input (Buffer)
		CROSS APPLY Deadlock.Process.nodes('executionStack') AS Execution (Frame)
	-- get the data from the executionStack node as XML
		CROSS APPLY (SELECT ExecutionStack = (SELECT   ProcNumber = ROW_NUMBER() 
									    OVER (PARTITION BY CTE.DeadlockID,
											       Deadlock.Process.value('@id', 'varchar(50)'),
											       Execution.Stack.value('@procname', 'sysname'),
											       Execution.Stack.value('@code', 'varchar(MAX)') 
										      ORDER BY (SELECT 1)),
								ProcName = Execution.Stack.value('@procname', 'sysname'),
								Line = Execution.Stack.value('@line', 'int'),
								SQLHandle = Execution.Stack.value('@sqlhandle', 'varchar(64)'),
								Code = LTRIM(RTRIM(Execution.Stack.value('.', 'varchar(MAX)')))
							FROM Execution.Frame.nodes('frame') AS Execution (Stack)
							ORDER BY ProcNumber
							FOR XML PATH('frame'), ROOT('executionStack'), TYPE )
			    ) es
        
/*************************************************************************************************
**
*************************************************************************************************/
	insert	into #tmpProcess
		(
		ProcessID		,	Victim			,	PageLockObject		,
		DeadLockObject		,	[Procedure]		,	LockMode		,
		Code			,	ClientApp		,	HostName		,
		LoginName		,	TransactionTime		,	InputBuffer		,
		associatedObjectID	,	[dbid]			,	fileid			,
		pageid			,	line			,	stmtstart		,
		stmtend			,	sqlhandle		,	drmID
		)
	select	p.ProcessID		,
		p.Victim		,
		n.PageLockObject	,
		n.DeadLockObject	,
		n.[Procedure]		,
		p.LockMode		,
		n.Code			,
		p.ClientApp		,
		p.HostName		,
		p.LoginName		,
		p.TransactionTime	,
		ltrim(rtrim(p.InputBuffer)) as InputBuffer,
		n.associatedObjectID	,
		n.[DBID]		,
		n.fileID		,
		n.pageID		,
		n.line			,
		n.stmtstart		,
		n.stmtend		,
		n.sqlhandle		,
		p.DeadlockID
	FROM    #Process p
		LEFT JOIN #Locks l ON p.DeadlockID = l.DeadlockID
		       AND p.ProcessID = l.LockProcessID
		left join #Lines n on n.DeadlockID = p.DeadlockID
			and p.ProcessID = n.ProcessID
	ORDER BY p.DeadlockId,
		p.Victim DESC,
		p.ProcessId;
/*************************************************************************************************
**
*************************************************************************************************/

	
	update	tp
	set	tp.[procedure] = (db_name(p.database_id) + '.' + OBJECT_SCHEMA_NAME ( p.obj_id, p.database_id ) + '.' + object_name( p.obj_id, p.database_id ))
	from	(
		select	distinct tmpProcessID, [procedure],
			ltrim(rtrim(left(right(ltrim(rtrim(InputBuffer)), (len(ltrim(rtrim(InputBuffer))) - (charindex('Database ID = ', ltrim(rtrim(InputBuffer))) + 13))), charindex(' ', right(ltrim(rtrim(InputBuffer)), (len(ltrim(rtrim(InputBuffer))) - (charindex('Database ID = ', ltrim(rtrim(InputBuffer))) + 13))))))) as database_id,
			ltrim(rtrim(left(right(ltrim(rtrim(InputBuffer)), (len(ltrim(rtrim(InputBuffer))) - (charindex('Object Id = ', ltrim(rtrim(InputBuffer))) + 11))), charindex(']', right(ltrim(rtrim(InputBuffer)), (len(ltrim(rtrim(InputBuffer))) - (charindex('Object Id = ', ltrim(rtrim(InputBuffer))) + 11))-1))))) as obj_id
		from	#tmpProcess p
		where	InputBuffer like '%Object ID%'
		) p
		inner join #tmpProcess tp on p.tmpProcessID = tp.tmpProcessID
	where	nullif(p.[procedure], '') is null

/*********************************************************************************************
**
*********************************************************************************************/
	insert	into #tmpLine
		(
		drmID		,	victim		,
		inputBuffer	,	processID	,
		LineList
		)
	SELECT	drmID				,
		victim				,
		ltrim(rtrim(inputBuffer))	,
		processID			,
		STUFF
			(
				(
				SELECT	distinct ',' + right(replicate('0', 5) + cast(line as varchar), 5)
				FROM	#tmpProcess
				WHERE	drmID = a.drmID
					and victim = a.victim
					and processID = a.processID
				order	by  ',' + right(replicate('0', 5) + cast(line as varchar), 5)
				FOR XML PATH ('')
				), 1, 1, ''
			)  AS LineList
	FROM	#tmpProcess AS a
	group	by
		drmID		,
		victim		,
		inputBuffer	,
		processID
/*********************************************************************************************
**
*********************************************************************************************/
	declare	@dbName			varchar(256)	,
		@dbID			varchar(20)	,
		@query			varchar(max)

		
	declare	c cursor for
	select	distinct
		db_name(p.[dbid]) as dbName	,
		p.[dbid]
	from	#tmpProcess p
	--
	open	c
	--
	fetch	next from c into @dbName, @dbID
		while	@@fetch_status != -1
			BEGIN
				set	@query = '
				update	p
				set	p.obj_name = (db_name(p.[dbid]) + ''.'' + OBJECT_SCHEMA_NAME ( t.object_id, p.[dbid] ) + ''.'' + object_name( t.object_id, p.[dbid] ) + isnull(''.'' + i.name, ''''))
				from	#tmpProcess p
					inner join ' + quotename(@dbname) + '.sys.partitions t	on t.partition_id = p.associatedObjectID
					left join ' + quotename(@dbname) + '.sys.indexes i	on i.object_id = t.object_id
						and i.index_id = t.index_id
				where	p.[dbid] = ' + char(39) + @dbid + char(39)
						
				exec	(@query)
				
				fetch	next from c into @dbName, @dbID
			END
	deallocate c

/*********************************************************************************************
**
*********************************************************************************************/
	--select	d.*
	--into	deadlock_report_detail_bak_20141204

	--delete	d
	--from	deadlock_report_detail d
	--	inner join #tmpProcess p on p.drmID = d.drmID

	--from	deadlock_report_detail d
	--	inner join #tmpProcess p on p.drmID = d.drmID


	insert	into deadlock_report_detail
		(
		dbName			,
		processID		,
		Victim			,
		pageLockObject		,
		deadLockObject		,
		[procedure]		,
		obj_name		,
		lockMode		,
		Code			,
		ClientApp		,
		HostName		,
		LoginName		,
		TransactionTime		,
		InputBuffer		,
		lines			,
		drmID
		)
	select	distinct
		DB_NAME(a.[dbid])	,
		a.processID		,
		a.victim		,
		a.pageLockObject	,
		a.deadLockObject	,
		a.[procedure]		,
		a.obj_name		,
		a.lockMode		,
		a.code			,
		a.clientApp		,
		a.HostName		,
		a.LoginName		,
		a.TransactionTime	,
		a.InputBuffer		,
		l.LineList		,
		a.drmID
	FROM	#tmpProcess a
		inner join #tmpLine l on l.drmID = a.drmID
			and l.processID = a.processID
/*********************************************************************************************
**
*********************************************************************************************/
GO
USE [DBA]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
if	object_id('tempdb..#tmpRes') is not null
		drop	table #tmpRes

	create	table #tmpRes
		(
		drmID		int		,
		processID	varchar(50)	,
		origInputBuffer	varchar(max)	,
		inputbuffer	varchar(max)
		)
		
		
if	object_id('tempdb..#tmpRes2') is not null
		drop	table #tmpRes2

	create	table #tmpRes2
		(
		drmID		int		,
		processID	varchar(50)	,
		origInputBuffer	varchar(max)	,
		inputbuffer	varchar(max)
		)

if	object_id('tempdb..#tmpDRD') is not null
		drop	table #tmpDRD
	
	CREATE	TABLE #tmpDRD
		(
		drmID		int		,
		PagelockObject	varchar(200)	,
		DeadlockObject	varchar(200)	,
		Victim		int		,
		[Procedure]	varchar(200)	,
		LockMode	char(1)		,
		Code		varchar(1000)	,
		ClientApp	varchar(100)	,
		HostName	varchar(20)	,
		LoginName	varchar(20)	,
		TransactionTime	datetime	,
		InputBuffer	varchar(max)	,
		event_data	xml		,
		obj_name	varchar(1024)	,
		lines		varchar(1024)	,
		processID	varchar(50)	,
		dbname		varchar(256)	
		)

if	object_id('tempdb..#tmpDRD_obfuscate') is not null
	drop	table #tmpDRD_obfuscate

CREATE	TABLE #tmpDRD_obfuscate
	(
	drmID		int		,
	PagelockObject	varchar(200)	,
	DeadlockObject	varchar(200)	,
	Victim		int		,
	[Procedure]	varchar(200)	,
	LockMode	char(1)		,
	Code		varchar(1000)	,
	ClientApp	varchar(100)	,
	HostName	varchar(20)	,
	LoginName	varchar(20)	,
	TransactionTime	datetime	,
	InputBuffer	varchar(max)	,
	event_data	xml		,
	obj_name	varchar(1024)	,
	lines		varchar(1024)	,
	processID	varchar(50)	,
	dbname		varchar(256)
	)
		
GO
if	object_id('uspAnalyzeDeadlocks_Obfuscate') is not null
	drop	procedure uspAnalyzeDeadlocks_Obfuscate
GO
/*************************************************************************************************
**  purpose:
**	Obfuscate query text
**  author:
**      William J. Wolf
**
**  date created:
**      2014-09-11
**
**  parameter description:

**  date modified/reason:
**
**  Notes:
DISCLAIMER:  This is written by William Wolf(@sqlwarewolf).  William is not responsible for any
adverse affects this may cause.  Always test thoroughly on a development system before implementing in production
**************************************************************************************************************/
CREATE	procedure [dbo].[uspAnalyzeDeadlocks_Obfuscate]
as
/**************************************************************************************************************
**
**************************************************************************************************************/

	--print	'uspAnalyzeDeadlocks_Obfuscate'

	set	nocount on
	
		
	declare	@drmID			int		,
		@processID		varchar(50)	,
		@origInputBuffer	varchar(max)	,
		@inputBuffer		varchar(max)
/**************************************************************************************************************
**
**************************************************************************************************************/	


	declare	c cursor for 
	select	drmID, processID, ltrim(rtrim(inputbuffer))
	from	#tmpDRD d
	where	charindex(char(39), inputbuffer) > 0

	--
	open	c
	--
	fetch	next from c into @drmID, @processID, @origInputBuffer
		while	@@fetch_status != -1
			BEGIN
				set	@inputBuffer = @origInputBuffer
				--print	@origInputBuffer
				while	charindex(char(39), @inputbuffer) > 0
					BEGIN
					
						set	@inputBuffer = left(@inputBuffer, charindex(char(39), @inputBuffer) - 1) + '?' +  
						right(right(@inputBuffer, len(@inputBuffer) - charindex(char(39), @inputBuffer)), len(right(@inputBuffer, len(@inputBuffer) - charindex(char(39), @inputBuffer))) - charindex(char(39), right(@inputBuffer, len(@inputBuffer) - charindex(char(39), @inputBuffer)))+1)
						
						set	@inputBuffer = replace(@inputBuffer, char(39) + '?' + char(39), '?')
						set	@inputBuffer = replace(@inputBuffer, '?' + char(39), '?')
						
						--print	@inputBuffer
					END
					
				insert	into #tmpRes(drmID, processID, origInputBuffer, inputBuffer)
				select	@drmID, @processID, @origInputBuffer, @inputBuffer

				set	@origInputBuffer = null


				fetch	next from c into @drmID, @processID, @origInputBuffer
			END
	deallocate c


/**************************************************************************************************************
**
**************************************************************************************************************/

	declare	c cursor for 
	select	d.drmID, d.processID, ltrim(rtrim(d.inputbuffer))
	from	#tmpDRD d
		left join #tmpRes r on r.drmID = d.drmID
			and r.processID = d.processID
	where	patindex('%[(, ,=,<,>,?][0-9]%', d.inputbuffer) > 0
		and r.drmID is null
		and d.inputBuffer not like '%Object ID%'
		--and d.drmID = 49
	union
	select	r.drmID, r.processID, ltrim(rtrim(r.inputbuffer))
	from	#tmpRes r
	where	patindex('%[(, ,=,<,>,?,.][0-9]%', r.inputbuffer) > 0
		--and r.drmID = 49

	--
	open	c
	--
	fetch	next from c into @drmID, @processID, @origInputBuffer
		while	@@fetch_status != -1
			BEGIN
				set	@inputBuffer = @origInputBuffer
				--print	@inputbuffer
				while	patindex('%[(, ,=,<,>,?,.][0-9]%', @inputbuffer) > 0
					BEGIN
					

						
						set	@inputBuffer = left(@inputbuffer, patindex('%[(, ,=,<,>,?,.][0-9]%', @inputbuffer) ) + '?' +
							right(@inputbuffer, len(@inputbuffer) - patindex('%[(, ,=,<,>,?,.][0-9]%', @inputbuffer)-1)
						--print	@inputbuffer
					END
					
				insert	into #tmpRes2(drmID, processID, origInputBuffer, inputBuffer)
				select	@drmID, @processID, @origInputBuffer, @inputBuffer

				set	@origInputBuffer = null


				fetch	next from c into @drmID, @processID, @origInputBuffer
			END
	deallocate c
	
/**************************************************************************************************************
**
**************************************************************************************************************/
	while	exists(select 1 from #tmpRes2 where inputBuffer like '%??%')
		BEGIN
			update	r
			set	r.inputBuffer = replace(r.inputBuffer, '??', '?')
			from	#tmpRes2 r
			where	r.inputBuffer like '%??%'
		END
		
	while	exists(select 1 from #tmpRes2 where inputBuffer like '%?,?%')
		BEGIN
			update	r
			set	r.inputBuffer = replace(r.inputBuffer, '?,?', '?')
			from	#tmpRes2 r
			where	r.inputBuffer like '%?,?%'
		END

	while	exists(select 1 from #tmpRes2 where inputBuffer like '%?, ?%')
		BEGIN
			update	r
			set	r.inputBuffer = replace(r.inputBuffer, '?, ?', '?')
			from	#tmpRes2 r
			where	r.inputBuffer like '%?, ?%'
		END
		
	while	exists(select 1 from #tmpRes2 where inputBuffer like '%?.?%')
		BEGIN
			update	r
			set	r.inputBuffer = replace(r.inputBuffer, '?.?', '?')
			from	#tmpRes2 r
			where	r.inputBuffer like '%?.?%'
		END
		
	update	r
	set	r.inputBuffer = replace(r.inputBuffer, 'NULL', '?')
	from	#tmpRes2 r
	where	r.inputBuffer like '%NULL%'

/**************************************************************************************************************
**
**************************************************************************************************************/
	insert	into #tmpDRD_obfuscate
		(
		drmID		,	PagelockObject	,
		DeadlockObject	,	Victim		,
		[Procedure]	,	LockMode	,
		Code		,	ClientApp	,
		HostName	,	LoginName	,
		TransactionTime	,	InputBuffer	,
		event_data	,	obj_name	,
		lines		,	processID	,
		dbName
		)
	
	select	d.drmID			,	d.PagelockObject	,
		d.DeadlockObject	,	d.Victim		,
		d.[Procedure]		,	d.LockMode		,
		d.Code			,	d.ClientApp		,
		d.HostName		,	d.LoginName		,
		d.TransactionTime	,	r.InputBuffer		,
		null as event_data	,	d.obj_name		,
		d.lines			,	d.processID		,
		d.dbName
	from	#tmpRes2 r
		inner join #tmpDRD d on r.drmID = d.drmID
			and r.processID = d.processID
	union
	select	d.drmID			,	d.PagelockObject	,
		d.DeadlockObject	,	d.Victim		,
		d.[Procedure]		,	d.LockMode		,
		d.Code			,	d.ClientApp		,
		d.HostName		,	d.LoginName		,
		d.TransactionTime	,	d.InputBuffer		,
		null as event_data	,	d.obj_name		,
		d.lines			,	d.processID		,
		d.dbName
	from	#tmpDRD d
		left join #tmpRes2 r on r.drmID = d.drmID
			and r.processID = d.processID
	where	r.drmID is null


GO
USE [DBA]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
if	object_id('uspAnalyzeDeadlocks') is not null
	drop	procedure uspAnalyzeDeadlocks
GO
/*************************************************************************************************
**  purpose:
**	report on deadlocks
**  author:
**      William J. Wolf
**
**  date created:
**      2014-09-11
**
**  parameter description:
**      @startDate
**	@endDate
**	@reportOption
**	@obfuscate

**  date modified/reason:
**
**  Notes:
DISCLAIMER:  This is written by William Wolf(@sqlwarewolf).  William is not responsible for any
adverse affects this may cause.  Always test thoroughly on a development system before implementing in production

1. Something to analyze the deadlocks (past hour/day/week/month/etc) and produce two lists. 
Independent of each other. Its just which object is involved in a deadlock and wins/loses the most. 


One list of the deadlock winner by order of most frequent to least frequent. (which objects win in a deadlock  they get to keep running) 
One list of the deadlock loser by order of most frequent to least frequent. (which objects loses out to a deadlock the most) 

2. Something to analyze the deadlocks (past hour/day/week/month/etc) and produce one list. 
This list shows the object combinations that are involved in deadlocks  what two objects collide the most often  by frequency. 

exec	uspAnalyzeDeadlocks @reportOption = 1, @startDate = '11/01/2014'
exec	uspAnalyzeDeadlocks @reportOption = 2

*************************************************************************************************/
CREATE	procedure [dbo].[uspAnalyzeDeadlocks]
	@startDate		datetime	= null		,
	@endDate		datetime	= null		,
	@reportOption		tinyint		= 1		,
	@obfuscate		bit		= 1		,
	@drmID			int		= null		,
	@objectName		varchar(256)	= null		,
	@includeLineNumbers	bit		= 0
as
/*******************************************************************************
**
*******************************************************************************/
	set	nocount on
	set	@startDate	= isnull(@startdate, dateadd(ww, -1, getdate()))
	set	@endDate	= isnull(@endDate, getdate())

	if	object_id('tempdb..#tmpDRM') is not null
		drop	table #tmpDRM
	
	create	 table #tmpDRM
		(
		drmID		int	,
		event_data	xml
		)
		
	create	clustered index #ix#tmpDRM on #tmpDRM(drmID)
	
	if	object_id('tempdb..#tmpDRD') is not null
		drop	table #tmpDRD
	
	CREATE	TABLE #tmpDRD
		(
		drmID		int		,
		PagelockObject	varchar(200)	,
		DeadlockObject	varchar(200)	,
		Victim		int		,
		[Procedure]	varchar(200)	,
		LockMode	varchar(10)	,
		Code		varchar(1000)	,
		ClientApp	varchar(100)	,
		HostName	varchar(20)	,
		LoginName	varchar(20)	,
		TransactionTime	datetime	,
		InputBuffer	varchar(max)	,
		event_data	xml		,
		obj_name	varchar(1024)	,
		lines		varchar(1024)	,
		processID	varchar(50)	,
		dbName		varchar(256)
		)
		
	create	clustered index #ix#tmpDRD on #tmpDRD(drmID)
	
	
	if	object_id('tempdb..#tmpDRD_obfuscate') is not null
		drop	table #tmpDRD_obfuscate
	
	CREATE	TABLE #tmpDRD_obfuscate
		(
		drmID		int		,
		PagelockObject	varchar(200)	,
		DeadlockObject	varchar(200)	,
		Victim		int		,
		[Procedure]	varchar(200)	,
		LockMode	varchar(10)	,
		Code		varchar(1000)	,
		ClientApp	varchar(100)	,
		HostName	varchar(20)	,
		LoginName	varchar(20)	,
		TransactionTime	datetime	,
		InputBuffer	varchar(max)	,
		event_data	xml		,
		obj_name	varchar(1024)	,
		lines		varchar(1024)	,
		processID	varchar(50)	,
		dbName		varchar(256)
		)
		
	create	clustered index #ix#tmpDRD_obfuscate on #tmpDRD_obfuscate(drmID)
	
	if	object_id('tempdb..#tmpDRD_Rep') is not null
		drop	table #tmpDRD_Rep
			
	CREATE	TABLE #tmpDRD_Rep
		(
		drmID			int				,
		nv_PagelockObject	varchar(200)			,
		nv_DeadlockObject	varchar(200)			,
		nv_Victim		int				,
		nv_Procedure		varchar(200)			,
		nv_LockMode		varchar(10)			,
		nv_Code			varchar(1000)			,
		nv_ClientApp		varchar(100)			,
		nv_HostName		varchar(20)			,
		nv_LoginName		varchar(20)			,
		nv_TransactionTime	datetime			,
		nv_InputBuffer		varchar(max)			,
		nv_obj_name		varchar(1024)			,
		nv_lines		varchar(1024)			,
		cs_nv_InputBuffer	as checksum(nv_InputBuffer)	,
		v_PagelockObject	varchar(200)			,
		v_DeadlockObject	varchar(200)			,
		v_Victim		int				,
		v_Procedure		varchar(200)			,
		v_LockMode		varchar(10)			,
		v_Code			varchar(1000)			,
		v_ClientApp		varchar(100)			,
		v_HostName		varchar(20)			,
		v_LoginName		varchar(20)			,
		v_TransactionTime	datetime			,
		v_InputBuffer		varchar(max)			,
		v_obj_name		varchar(1024)			,
		v_lines			varchar(1024)			,
		cs_v_InputBuffer	as checksum(nv_InputBuffer)	,
		dbName			varchar(256)
		)
		
	create	clustered index #ix#tmpDRD_Rep on #tmpDRD_Rep(nv_DeadlockObject, v_DeadlockObject)
	create	index #ix#tmpDRD_Rep002 on #tmpDRD_Rep(cs_nv_InputBuffer, cs_v_InputBuffer)
	
	
	if	object_id('tempdb..#tmpRes') is not null
		drop	table #tmpRes

	create	table #tmpRes
		(
		drmID		int		,
		processID	varchar(50)	,
		origInputBuffer	varchar(max)	,
		inputbuffer	varchar(max)
		)
		
		
	if	object_id('tempdb..#tmpRes2') is not null
		drop	table #tmpRes2

	create	table #tmpRes2
		(
		drmID		int		,
		processID	varchar(50)	,
		origInputBuffer	varchar(max)	,
		inputbuffer	varchar(max)
		)
	
/*******************************************************************************
**
*******************************************************************************/
	insert	into #tmpDRM(drmID, event_data)
	select	drmID, event_data
	from	Deadlock_Report_Main m
	where	exists(select 1 from Deadlock_Report_Detail d where d.TransactionTime between @startDate and @endDate and d.drmID = m.drmID)
	--where	drmDate between @startDate and @endDate


	insert	into #tmpDRD
		(
		drmID		,
		PagelockObject	,
		DeadlockObject	,
		Victim		,
		[Procedure]	,
		LockMode	,
		Code		,
		ClientApp	,
		HostName	,
		LoginName	,
		TransactionTime	,
		InputBuffer	,
		event_data	,
		obj_name	,
		lines		,
		processID	,
		dbName
		)
	select	distinct
		d.drmID				,
		d.PagelockObject		,
		d.DeadlockObject		,
		d.Victim			,
		d.[Procedure]			,
		d.LockMode			,
		d.Code				,
		d.ClientApp			,
		d.HostName			,
		d.LoginName			,
		d.TransactionTime		,
		ltrim(rtrim(
		replace(replace(replace(replace(d.InputBuffer, char(13) + char(10), ' '), char(10), ' '), char(13), ' '), '	', ' ')
		))				,
		null,--m.event_data			,
		d.obj_name			,
		case	@includeLineNumbers
			when	1 then d.lines
			else	null
		end				,
		d.processID			,
		d.dbName
	from	#tmpDRM m
		inner join Deadlock_Report_Detail d on d.drmID = m.drmID
	
	--where	m.drmID in (84696)
/*******************************************************************************
**
*******************************************************************************/
	if	@obfuscate = 1
		BEGIN
			exec	uspAnalyzeDeadlocks_Obfuscate
			
		
			insert	into #tmpDRD_Rep
				(
				drmID			,	nv_PagelockObject	,
				nv_DeadlockObject	,	nv_Victim		,
				nv_Procedure		,	nv_LockMode		,
				nv_Code			,	nv_ClientApp		,
				nv_HostName		,	nv_LoginName		,
				nv_TransactionTime	,	nv_InputBuffer		,
				nv_obj_name		,	nv_lines		,
				v_PagelockObject	,	v_DeadlockObject	,
				v_Victim		,	v_Procedure		,
				v_LockMode		,	v_Code			,
				v_ClientApp		,	v_HostName		,
				v_LoginName		,	v_TransactionTime	,
				v_InputBuffer		,	v_obj_name		,
				v_lines			,	dbName
				)
			select	distinct
				nv.drmID		as	drmID			,
				nv.PageLockObject	as	nv_PagelockObject	,
				nv.DeadlockObject	as	nv_DeadlockObject	,
				nv.Victim		as	nv_Victim		,
				nv.[Procedure]		as	nv_Procedure		,
				nv.LockMode		as	nv_LockMode		,
				nv.Code			as	nv_Code			,
				nv.ClientApp		as	nv_ClientApp		,
				nv.HostName		as	nv_HostName		,
				nv.LoginName		as	nv_LoginName		,
				nv.TransactionTime	as	nv_TransactionTime	,
				nv.InputBuffer		as	nv_InputBuffer		,
				nv.obj_name		as	nv_obj_name		,
				nv.lines		as	nv_lines		,
				v.PagelockObject	as	v_PagelockObject	,
				v.DeadlockObject	as	v_DeadlockObject	,
				v.Victim		as	v_Victim		,
				v.[Procedure]		as	v_Procedure		,
				v.LockMode		as	v_LockMode		,
				v.Code			as	v_Code			,
				v.ClientApp		as	v_ClientApp		,
				v.HostName		as	v_HostName		,
				v.LoginName		as	v_LoginName		,
				v.TransactionTime	as	v_TransactionTime	,
				v.InputBuffer		as	v_InputBuffer		,
				v.obj_name		as	v_obj_name		,
				v.lines			as	v_lines			,
				v.dbName		as	dbName
			from	#tmpDRD_obfuscate nv
				inner join #tmpDRD_obfuscate v on v.drmID = nv.drmID
			where	nv.Victim	= 0
				and v.Victim	= 1
		END
	else
/*******************************************************************************
**
*******************************************************************************/
		BEGIN
			insert	into #tmpDRD_Rep
				(
				drmID			,	nv_PagelockObject	,
				nv_DeadlockObject	,	nv_Victim		,
				nv_Procedure		,	nv_LockMode		,
				nv_Code			,	nv_ClientApp		,
				nv_HostName		,	nv_LoginName		,
				nv_TransactionTime	,	nv_InputBuffer		,
				nv_obj_name		,	nv_lines		,
				v_PagelockObject	,	v_DeadlockObject	,
				v_Victim		,	v_Procedure		,
				v_LockMode		,	v_Code			,
				v_ClientApp		,	v_HostName		,
				v_LoginName		,	v_TransactionTime	,
				v_InputBuffer		,	v_obj_name		,
				v_lines			,	dbName
				)
			select	distinct
				nv.drmID		as	drmID			,
				nv.PageLockObject	as	nv_PagelockObject	,
				nv.DeadlockObject	as	nv_DeadlockObject	,
				nv.Victim		as	nv_Victim		,
				nv.[Procedure]		as	nv_Procedure		,
				nv.LockMode		as	nv_LockMode		,
				nv.Code			as	nv_Code			,
				nv.ClientApp		as	nv_ClientApp		,
				nv.HostName		as	nv_HostName		,
				nv.LoginName		as	nv_LoginName		,
				nv.TransactionTime	as	nv_TransactionTime	,
				nv.InputBuffer		as	nv_InputBuffer		,
				nv.obj_name		as	nv_obj_name		,
				nv.lines		as	nv_lines		,
				v.PagelockObject	as	v_PagelockObject	,
				v.DeadlockObject	as	v_DeadlockObject	,
				v.Victim		as	v_Victim		,
				v.[Procedure]		as	v_Procedure		,
				v.LockMode		as	v_LockMode		,
				v.Code			as	v_Code			,
				v.ClientApp		as	v_ClientApp		,
				v.HostName		as	v_HostName		,
				v.LoginName		as	v_LoginName		,
				v.TransactionTime	as	v_TransactionTime	,
				v.InputBuffer		as	v_InputBuffer		,
				v.obj_name		as	v_obj_name		,
				v.lines			as	v_lines			,
				v.dbName		as	dbName
			from	#tmpDRD nv
				inner join #tmpDRD v on v.drmID = nv.drmID
			where	nv.Victim	= 0
				and v.Victim	= 1
				
		END

	--select	*
	--from	#tmpDRD_obfuscate

	--select	*
	--from	#tmpDRD_Rep

/*******************************************************************************
** Winners/losers by object
*******************************************************************************/
set	@objectName = '%' + @objectName + '%'

if	@reportOption = 1
	BEGIN
/*******************************************************************************
** Winners/losers by query
*******************************************************************************/	
	
	if	@objectName is null
		BEGIN
			select	nv_obj_name		as winner_object	,
				isnull(nullif(nv_Procedure, ''), nv_InputBuffer)		as 'winner procedure/query'		,
				nv_lines	as winner_lines	,
				max(nv_TransactionTime)	as LastTransactionTime	,
				count(drmID)		as Frequency		,
				dbName
			from	#tmpDRD_Rep
			where	nullif(isnull(nullif(nv_Procedure, ''), nv_InputBuffer), '') is not null
			group	by
				nv_obj_name, isnull(nullif(nv_Procedure, ''), nv_InputBuffer), nv_lines, dbName
			order	by 5 desc
			
			select	v_obj_name		as loser_object	,
				isnull(nullif(v_Procedure, ''), v_InputBuffer)		as 'loser procedure/query'		,
				v_lines	as loser_lines,
				max(v_TransactionTime)	as LastTransactionTime	,
				count(drmID)		as Frequency		,
				dbName
			from	#tmpDRD_Rep
			where	nullif(isnull(nullif(v_Procedure, ''), v_InputBuffer), '') is not null
			group	by
				v_obj_name, isnull(nullif(v_Procedure, ''), v_InputBuffer), v_lines, dbName
			order	by 5 desc
		END
	else
		BEGIN

		
			select	nv_obj_name		as winner_object	,
				isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer)		as 'winner procedure/query'		,
				nv_lines	as winner_lines	,
				max(nv_TransactionTime)	as LastTransactionTime	,
				count(drmID)		as Frequency		,
				dbName
			from	#tmpDRD_Rep
			where	nullif(isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer), '') like @objectName
				or nv_obj_name like @objectName 
			group	by
				nv_obj_name, isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer), nv_lines, dbName
			order	by 5 desc
			
			select	v_obj_name		as loser_object	,
				isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer)		as 'loser procedure/query'		,
				v_lines	as loser_lines,
				max(v_TransactionTime)	as LastTransactionTime	,
				count(drmID)		as Frequencey		,
				dbName
			from	#tmpDRD_Rep
			where	nullif(isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer), '') like @objectName
				or v_obj_name like @objectName
			group	by
				v_obj_name, isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer), v_lines, dbName
			order	by 5 desc
		END
	------------------
	END
/*******************************************************************************
** both by object
*******************************************************************************/
else	if	@reportOption = 2
	BEGIN

	
	if	@objectName is null
		BEGIN
			select	distinct *
			from	(
				select	nv_obj_name		as winner_object	,
					isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer)		as 'winner procedure/query'		,
					max(nv_TransactionTime)	as LastTransactionTime		,
					nv_lines	as winner_lines,
					v_obj_name		as loser_object	,
					isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer)		as 'loser procedure/query'		,
					v_lines		as loser_lines,
					max(v_TransactionTime)	as loser_LastTransactionTime	,
					count(drmID)		as Frequency			,
					dbName
				from	#tmpDRD_Rep
				where	nullif(isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer), '') is not null
					and nullif(isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer), '') is not null
				group	by
					nv_obj_name,
					v_obj_name,
					isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer),
					isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer),
					nv_lines,
					v_lines,
					dbName
				) a
			order	by 9 desc
		END
	else
		BEGIN
			select	distinct *
			from	(
				select	nv_obj_name		as winner_object	,
					isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer)		as 'winner procedure/query'		,
					max(nv_TransactionTime)	as LastTransactionTime		,
					nv_lines	as winner_lines,
					v_obj_name		as loser_object	,
					isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer)		as 'loser procedure/query'		,
					v_lines		as loser_lines,
					max(v_TransactionTime)	as loser_LastTransactionTime	,
					count(drmID)		as Frequency			,
					dbName
				from	#tmpDRD_Rep
				where	nullif(isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer), '') is not null
					and nullif(isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer), '') is not null
				group	by
					nv_obj_name,
					v_obj_name,
					isnull(nullif(cast(nv_Procedure as varchar(max)), ''), nv_InputBuffer),
					isnull(nullif(cast(v_Procedure as varchar(max)), ''), v_InputBuffer),
					nv_lines,
					v_lines,
					dbName
				) a
			where	winner_object like @objectName
				or loser_object like @objectName
				or [winner procedure/query] like @objectName
				or [loser procedure/query] like @objectName
			order	by 9 desc
		END
		------------------
	END




GO
USE [DBA]
GO
if	object_id('uspCleanDeadlockHistory') is not null
	drop	procedure uspCleanDeadlockHistory

GO
/*************************************************************************************************
**  purpose:
**	clean deadlock reporting history
**  author:
**      William J. Wolf
**
**  date created:
**      2014-09-11
**
**  parameter description:
	@eventDate
	@daysToKeep

**  date modified/reason:
**
**  Notes:
DISCLAIMER:  This is written by William Wolf(@sqlwarewolf).  William is not responsible for any
adverse affects this may cause.  Always test thoroughly on a development system before implementing in production
**************************************************************************************************************/
CREATE	procedure [dbo].[uspCleanDeadlockHistory]
	@eventDate	datetime = null	,
	@daysToKeep	smallint = null
as
/*******************************************************************************
**
*******************************************************************************/
	SET	XACT_ABORT on
	set	nocount on
	
	if	@eventDate is null
		and @daysToKeep is null
		BEGIN
			raisError('Provide either @eventDate or @daysToKeep', 16, 1)
			return
		END
		
	if	@daysToKeep is not null
		and @eventDate is null
		BEGIN
			set	@daysToKeep	= abs(@daysToKeep) * -1
			set	@eventDate	= isnull(@eventDate, dateadd(dd, @daysToKeep, getdate()))
		END


	if	object_id('tempdb..#tmpDRM') is not null
		drop	table #tmpDRM
	
	create	 table #tmpDRM
		(
		drmID		int	,
		event_data	xml
		)
		
	create	clustered index #ix#tmpDRM on #tmpDRM(drmID)
	
/*******************************************************************************
**
*******************************************************************************/
	insert	into #tmpDRM(drmID)
	select	drmID
	from	Deadlock_Report_Main
	where	drmDate <= @eventDate
	
	delete	d
	from	Deadlock_Report_Detail d
		inner join #tmpDRM m on m.drmID = d.drmID
		
	
	delete	d
	from	Deadlock_Report_Main d
		inner join #tmpDRM m on m.drmID = d.drmID
GO
USE [msdb]
GO
IF  EXISTS (SELECT job_id FROM msdb.dbo.sysjobs_view WHERE name = N'Record_Deadlocks')
EXEC msdb.dbo.sp_delete_job @job_name=N'Record_Deadlocks', @delete_unused_schedule=1
GO

USE [msdb]
GO
BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0
IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'[DB Maintenance]' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'[DB Maintenance]'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N'Record_Deadlocks', 
		@enabled=1, 
		@notify_level_eventlog=0, 
		@notify_level_email=0, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'No description available.', 
		@category_name=N'[DB Maintenance]', 
		@owner_login_name=N'sa', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Record_Deadlocks', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=3, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'exec	uspRecordDeadlocks', 
		@database_name=N'DBA', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Clean_Deadlock_History', 
		@step_id=2, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'declare	@eventDate	datetime = dateadd(dd, -90, getdate())


exec	uspCleanDeadlockHistory @eventDate = @eventDate', 
		@database_name=N'DBA', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N'Record_Deadlocks', 
		@enabled=1, 
		@freq_type=4, 
		@freq_interval=1, 
		@freq_subday_type=4, 
		@freq_subday_interval=5, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=0, 
		@active_start_date=20140910, 
		@active_end_date=99991231, 
		@active_start_time=0, 
		@active_end_time=235959, 
		@schedule_uid=N'3a293675-20bc-4486-85b6-9ba6e250d5d2'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:

GO


